/**
 *  Copyright 2009-2010 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 *
 *  WARNING: this file is here for development / testing purposes only
 *  and its contents and input / output can change without any prior
 *  notice.
 */

#ifndef MIRROR_DEVEL_TEST_1011291901_HPP
#define MIRROR_DEVEL_TEST_1011291901_HPP

#include <mirror/mirror_base.hpp>
#include <mirror/utils/quick_reg.hpp>

namespace test {

struct x
{
	int a;
	bool b;
	long c;
	double d;
	long double e;
	float f;

	x(void)
	 : a(0), b(1), c(2), d(3.4), e(5.6), f(7.8f)
	{ }

	x(int _a)
	 : a(_a), b(1), c(2), d(3.4), e(5.6), f(7.8f)
	{ }

	x(int _a, bool _b)
	 : a(_a), b(_b), c(2), d(3.4), e(5.6), f(7.8f)
	{ }

	x(int _a, bool _b, long _c)
	 : a(_a), b(_b), c(_c), d(3.4), e(5.6), f(7.8f)
	{ }

	x(const std::tm& t)
	 : a(0), b(1), c(2), d(3.4), e(5.6), f(7.8f)
	{ }

	double func(int u, int v) const
	{
		return a ? (u + v) / double(a) : 0;
	}
};

struct y
{
	x g;
	x h;

	y(void)
	{ }

	y(const x& _g, const x& _h)
	 : g(_g)
	 , h(_h)
	{ }
};

struct z
{
	int a;
	bool b;
	char c;
	double d, e;
	float f;
};

struct foo
{
	struct bar
	{
		int baz;
	} fubar;
};

enum class gender
{
	unspecified, male, female
};

struct person
{
	enum class salutation
	{
		mr, mrs, miss, dr
	} title;

	std::string first_name;
	std::string middle_name;
	std::string family_name;

	std::tm birth_date;
	gender sex;

	float weight;
	float height;

	person(
		salutation _title,
		const std::string& _first_name,
		const std::string& _family_name,
		const std::tm _birth_date,
		gender _sex,
		float _weight,
		float _height
	): title(_title)
	 , first_name(_first_name)
	 , family_name(_family_name)
	 , birth_date(_birth_date)
	 , sex(_sex)
	 , weight(_weight)
	 , height(_height)
	{ }

	person(
		salutation _title,
		const std::string& _first_name,
		const std::string& _middle_name,
		const std::string& _family_name,
		const std::tm _birth_date,
		gender _sex,
		float _weight,
		float _height
	): title(_title)
	 , first_name(_first_name)
	 , middle_name(_middle_name)
	 , family_name(_family_name)
	 , birth_date(_birth_date)
	 , sex(_sex)
	 , weight(_weight)
	 , height(_height)
	{ }
};

} // namespace test

MIRROR_REG_BEGIN

MIRROR_QREG_GLOBAL_SCOPE_NAMESPACE(test)

MIRROR_REG_CLASS_BEGIN(struct, test, x)
MIRROR_REG_CLASS_MEM_VARS_BEGIN
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, a)
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, b)
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, c)
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, d)
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, e)
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, f)
MIRROR_REG_CLASS_MEM_VARS_END
MIRROR_REG_CONSTRUCTORS_BEGIN
	MIRROR_REG_DEFAULT_CONSTRUCTOR(public)
	MIRROR_REG_COPY_CONSTRUCTOR(public)

	MIRROR_REG_CONSTRUCTOR_BEGIN(public, a)
		MIRROR_REG_CONSTRUCTOR_PARAM(int, a)
	MIRROR_REG_CONSTRUCTOR_END(a)

	MIRROR_REG_CONSTRUCTOR_BEGIN(public, ab)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, a)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, b)
	MIRROR_REG_CONSTRUCTOR_END(ab)

	MIRROR_REG_CONSTRUCTOR_BEGIN(public, abc)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, a)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, b)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, c)
	MIRROR_REG_CONSTRUCTOR_END(abc)

	MIRROR_REG_CONSTRUCTOR_BEGIN(public, t)
		MIRROR_REG_CONSTRUCTOR_PARAM(std::tm, t)
	MIRROR_REG_CONSTRUCTOR_END(t)
MIRROR_REG_CONSTRUCTORS_END
MIRROR_REG_MEM_FUNCTIONS_BEGIN
	MIRROR_REG_MEM_FUNCTION_BEGIN(public,_,double,func,_)
		MIRROR_REG_MEM_FUNCTION_PARAM(int, u)
		MIRROR_REG_MEM_FUNCTION_PARAM(int, v)
	MIRROR_REG_MEM_FUNCTION_END(func,_,const)
MIRROR_REG_MEM_FUNCTIONS_END
MIRROR_REG_CLASS_END

MIRROR_REG_CLASS_BEGIN(struct, test, y)
MIRROR_REG_CLASS_MEM_VARS_BEGIN
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, g)
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, h)
MIRROR_REG_CLASS_MEM_VARS_END
MIRROR_REG_CONSTRUCTORS_BEGIN
	MIRROR_REG_DEFAULT_CONSTRUCTOR(public)
	MIRROR_REG_COPY_CONSTRUCTOR(public)

	MIRROR_REG_CONSTRUCTOR_BEGIN(public, gh)
		MIRROR_REG_CONSTRUCTOR_PARAM(test::x, g)
		MIRROR_REG_CONSTRUCTOR_PARAM(test::x, h)
	MIRROR_REG_CONSTRUCTOR_END(gh)
MIRROR_REG_CONSTRUCTORS_END
MIRROR_REG_CLASS_END

MIRROR_QREG_POD_CLASS(struct, test, z, (a)(b)(c)(d)(e)(f))

MIRROR_REG_CLASS_BEGIN(struct, test, foo)
MIRROR_REG_CLASS_MEM_VARS_BEGIN
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, fubar)
MIRROR_REG_CLASS_MEM_VARS_END
MIRROR_REG_CLASS_END

MIRROR_REG_NESTED_CLASS_BEGIN(struct, test::foo, bar)
MIRROR_REG_CLASS_MEM_VARS_BEGIN
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, baz)
MIRROR_REG_CLASS_MEM_VARS_END
MIRROR_REG_CLASS_END

MIRROR_REG_ENUM_BEGIN(test, gender)
MIRROR_REG_ENUM_VALUE(unspecified)
MIRROR_REG_ENUM_VALUE(male)
MIRROR_REG_ENUM_VALUE(female)
MIRROR_REG_ENUM_END

MIRROR_REG_CLASS_BEGIN(struct, test, person)
MIRROR_REG_CLASS_MEM_VARS_BEGIN
	MIRROR_REG_CLASS_MEM_VAR_CTS(_, _, _, title, ('t','i','t','l','e'))
	MIRROR_REG_CLASS_MEM_VAR_CTS(_, _, _, first_name, ('f','i','r','s','t','_','n','a','m','e'))
	MIRROR_REG_CLASS_MEM_VAR_CTS(_, _, _, middle_name, ('m','i','d','d','l','e','_','n','a','m','e'))
	MIRROR_REG_CLASS_MEM_VAR_CTS(_, _, _, family_name, ('f','a','m','i','l','y','_','n','a','m','e'))
	MIRROR_REG_CLASS_MEM_VAR_CTS(_, _, _, birth_date, ('b','i','r','t','h','_','d','a','t','e'))
	MIRROR_REG_CLASS_MEM_VAR_CTS(_, _, _, sex, ('s','e','x'))
	MIRROR_REG_CLASS_MEM_VAR_CTS(_, _, _, weight, ('w','e','i','g','h','t'))
	MIRROR_REG_CLASS_MEM_VAR_CTS(_, _, _, height, ('h','e','i','g','h','t'))
MIRROR_REG_CLASS_MEM_VARS_END
MIRROR_REG_CONSTRUCTORS_BEGIN
	MIRROR_REG_COPY_CONSTRUCTOR(public)

	MIRROR_REG_CONSTRUCTOR_BEGIN(public, 1)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, title)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, first_name)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, family_name)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, birth_date)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, sex)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, weight)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, height)
	MIRROR_REG_CONSTRUCTOR_END(1)

	MIRROR_REG_CONSTRUCTOR_BEGIN(public, 2)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, title)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, first_name)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, middle_name)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, family_name)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, birth_date)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, sex)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, weight)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, height)
	MIRROR_REG_CONSTRUCTOR_END(2)
MIRROR_REG_CONSTRUCTORS_END
MIRROR_REG_CLASS_END

MIRROR_REG_NESTED_ENUM_BEGIN(test::person, salutation)
MIRROR_REG_ENUM_VALUE(mr)
MIRROR_REG_ENUM_VALUE(mrs)
MIRROR_REG_ENUM_VALUE(miss)
MIRROR_REG_ENUM_VALUE(dr)
MIRROR_REG_ENUM_END

MIRROR_REG_END

#endif

