/**
 *  Copyright 2009-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 *
 *  WARNING: this file is here for development / testing purposes only
 *  and its contents and input / output can change witout any prior
 *  notice.
 */

#include <mirror/mirror.hpp>
#include <mirror/stream.hpp>
#include <mirror/factory.hpp>
#include <mirror/using_directive.hpp>
#include <mirror/utils/script_factory.hpp>
#include <mirror/utils/mijson.hpp>
#include <mirror/by_name.hpp>
#include <mirror/class_gen.hpp>

#include <cassert>
#include <iostream>
#include <stdexcept>
#include <functional>
#include <ctime>

#include "./test.hpp"

MIRROR_NAMESPACE_BEGIN

MIRROR_NAMESPACE_END

template <class MetaClass>
struct class_transf
{
	struct type
	{
		template <typename T>
		type(const T&){ }
	};
};

template <class MetaVariable>
struct mem_var_transf
{
	class type
	{
	private:
		typedef typename MetaVariable::type::original_type orig_type;
		mirror::mijson_element<const char*>& node;
	public:
		template <class TransfClass, class NodeRef>
		type(TransfClass&, NodeRef node)
		 : node(node.get())
		{ }

		orig_type get(void) const
		{
			mirror::lex_conv<std::string, orig_type> conv;
			return conv(
				node.member(
					MetaVariable::base_name()
				).value()
			);
		}
	};
};

int main(void)
{
	try
	{
		using namespace mirror;
		MIRROR_USING_NAMESPACE(test);
		//
		std::string src =
			"person("\
			"	dr, "\
			"	'\\u0414\\u043C\\u0438\\u0442\\u0440\\u0438\\u0439', "\
			"	'\\u0418\\u0432\\u0430\\u043D\\u043E\\u0432\\u0438\\u0446', "\
			"	'\\u041C\\u0435\\u043D\\u0414\\u0435\\u043B\\u0435\\u0435\\u0432', "\
			"	'1834-02-08', "\
			"	male, "\
			"	75, "\
			"	175"\
			")";
		//
		script_factory_input in;
		script_factory_maker::factory<test::person>::type f = in.data();
		in.set(src);
		test::person p(f());
		std::cout << stream::to_json::from(
			p,[](std::ostream& out){ out << "p"; }
		) << std::endl;
		//
		typedef member_variables_by_name<reflected_type<test::person> >::family_name meta_mv;
		std::cout << *meta_mv::address(p) << std::endl;
		std::string test::person::*pstr = meta_mv::member_pointer();
		std::cout << p.*pstr << std::endl;
		//
		const char json[] = " \
				{ \
					'title': 'mr', \
					'first_name': 'Johnny', \
					'middle_name': 'B.', \
					'family_name': 'Goode', \
					'birth_date': '1906-01-02', \
					'sex': 'male', \
					'weight': 75, \
					'height': 170 \
				} \
			";
		mirror::mijson_fragment<const char*> frag(
			json,
			json + sizeof(json) - 1
		);
		//
		//
		typedef generator_maker<class_transf, mem_var_transf> make;
		make::generator<test::person>::type tp(std::ref(frag));
		std::cout <<
			tp.first_name.get() << " " <<
			tp.family_name.get() << ", height: " <<
			tp.height.get() << "." <<
			std::endl;
		//
		//
		std::cout << mp::front<member_variables<reflected<test::foo> > >::type::full_name() << std::endl;
		std::cout << mp::front<member_variables<reflected<test::foo::bar> > >::type::full_name() << std::endl;
	}
	catch(std::exception const& error)
	{
		std::cerr << "Error: " << error.what() << std::endl;
	}
	//
	return 0;
}
