/**
 *  Copyright 2009-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 *
 *  WARNING: this file is here for development / testing purposes only
 *  and its contents and input / output can change witout any prior
 *  notice.
 */

#include <mirror/mirror.hpp>
#include <puddle/puddle.hpp>

#define RUBBER_META_TYPE_OPERATIONS write_to_cout, read_from_cin

#include <rubber/operation/write_to_ostream.hpp>
#include <rubber/operation/read_from_istream.hpp>

//#define RUBBER_POLYMORPHIC_META_OBJECT 1

#include <rubber/rubber.hpp>

#include <stdexcept>

#include "./test.hpp"

RUBBER_NAMESPACE_BEGIN

RUBBER_NAMESPACE_END

/*
void print_scope(const rubber::meta_scope& scope)
{
	using namespace rubber;
	auto r = scope.members();
	while(!r.empty())
	{
		std::cout <<
			r.front().self().construct_name() <<
			" " <<
			r.front().base_name();
		auto &mnso = r.front();
		if(mnso.is_type())
		{
			std::cout << ", constructors: ";
			auto c = mnso.as<meta_type>().constructors();
			while(!c.empty())
			{
				std::cout << c.front().base_name() << "(";
				auto p = c.front().parameters();
				while(!p.empty())
				{
					std::cout <<
						p.front().base_name();
					p.step_front();
					if(!p.empty())
						std::cout << ", ";
				}
				std::cout << ")";
				c.step_front();
				if(!c.empty()) std::cout << ", ";
			}
			raw_ptr x = mnso.as<meta_type>().new_();
			if(x)
			{
				mnso.as<meta_type>().delete_(
					mnso.as<meta_type>().new_copy(x)
				);
			}
			mnso.as<meta_type>().delete_(x);
		}
		std::cout << std::endl;
		if(mnso.is_scope())
			print_scope(mnso.as<meta_scope>());
		r.step_front();
	}
}
*/

int main(void)
{
	try
	{
		unsigned val = 12345;
		//
		mirror::mp::for_each<mirror::members<MIRRORED_GLOBAL_SCOPE()>>(
			rubber::eraser<
				rubber::meta_namespace,
				rubber::meta_type
			>(
				[](const rubber::meta_namespace& ns)
				{
					std::cout << "namespace " << ns.base_name() << std::endl;
				},
				[](const rubber::meta_type& typ)
				{
					std::cout << "type " << typ.base_name() << std::endl;
				}
			)
		);
		//
		auto x = puddle::adapt<mirror::reflected<decltype(val)>::type>();
		auto e = rubber::erase_type(x);
		std::cout << e.base_name() << std::endl;
		std::cout << (e.is_type() ? "true" : "false") << std::endl;
		std::cout << (e.scope().is_type() ? "true" : "false") << std::endl;
		std::cout << e.scope().members().size() << std::endl;
		//
		//print_scope(e.scope());
		std::cout << std::endl;
		//
		rubber::some bla = rubber::some(val, std::true_type());
		std::cout << bla.as<const unsigned>() << std::endl;
		//
		e.scope().members().for_each(
			[](const rubber::meta_named_object& mo)
			{
				std::cout <<
					mo.self().construct_name() <<
					" " <<
					mo.base_name() <<
					std::endl;
			}
		);
		//
		assert(e.can_examine_by<rubber::write_to_cout>());
		e.examine_by<rubber::write_to_cout>(rubber::some(val));
		std::cout << std::endl;
		{
			auto ref = rubber::some_ref(val);
			e.modify_by<rubber::read_from_cin>(ref);
		}
		e.examine_by<rubber::write_to_cout>(rubber::some(val));
		std::cout << std::endl;
		//
		test::person johnny = {
			test::person::salutation::mr,
			"John", "B.", "Goode",
			std::tm(),
			test::gender::male,
			170.f, 80.f
		};
		rubber::some pe = rubber::some(johnny, std::true_type());
		auto xp = puddle::adapt<mirror::reflected<test::person>::type>();
		auto ep = rubber::erase_type(xp);
		ep.member_variables().at(1).set(pe, std::string("Johnny"));
		std::cout << ep.member_variables().at(1).get(pe).as<const std::string>() << std::endl;
		std::cout << ep.member_variables().at(3).get(pe).as<const std::string>() << std::endl;
	}
	catch(std::exception const& error)
	{
		std::cerr << "Error: " << error.what() << std::endl;
	}
	//
	return 0;
}
