/**
 * @file rubber/config.hpp
 * @brief Configuration options
 *
 *  Copyright 2008-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef RUBBER_CONFIG_1103151301_HPP
#define RUBBER_CONFIG_1103151301_HPP

#include <mirror/raw_ptr/raw_ptr.hpp>
#include <mirror/utils/some_type.hpp>

// Macro that begins the Rubber library namespace
#define RUBBER_NAMESPACE_BEGIN \
namespace rubber {

// Macro which ends the Rubber library namespace
#define RUBBER_NAMESPACE_END \
} /* namespace rubber */

#ifndef RUBBER_POLYMORPHIC_META_OBJECT
#define RUBBER_POLYMORPHIC_META_OBJECT 0
#endif

#ifndef RUBBER_NO_NAMESPACE_MEMBERS
#define RUBBER_NO_NAMESPACE_MEMBERS 0
#endif

#ifndef RUBBER_MNSO_WITH_LOCAL_NAME
#define RUBBER_MNSO_WITH_LOCAL_NAME 0
#endif

#ifndef RUBBER_META_TYPE_OPERATIONS
#define RUBBER_META_TYPE_OPERATIONS
#endif

RUBBER_NAMESPACE_BEGIN

using mirror::raw_ptr;
using mirror::raw_cast;

using mirror::some;
using mirror::some_ref;
using mirror::some_cast;

namespace aux {

inline int& meta_object_uid_seq(void)
{
	static int seq = 0;
	return seq;
}

template <typename MirrorMetaObject>
inline int meta_object_uid(MirrorMetaObject)
{
	static int uid = ++meta_object_uid_seq();
	return uid;
}

template <typename TypeErased>
struct return_value
{
#if RUBBER_POLYMORPHIC_META_OBJECT
	typedef const TypeErased& type;
#else
	typedef TypeErased type;
#endif
};

} // namespace aux

RUBBER_NAMESPACE_END

#endif //include guard

