/**
 * @file rubber/meta_object.hpp
 * @brief Wrapper for meta-object
 *
 *  Copyright 2008-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef RUBBER_META_OBJECT_1103151301_HPP
#define RUBBER_META_OBJECT_1103151301_HPP

#include <rubber/config.hpp>
#include <rubber/meta_meta_object.hpp>
#include <mirror/meta_object_tags.hpp>

RUBBER_NAMESPACE_BEGIN

class meta_object
{
private:
	int _uid;
	aux::return_value<meta_meta_object>::type (*_get_self)(void);
public:
	template <typename MetaObject>
	meta_object(MetaObject mo)
	 : _uid(aux::meta_object_uid(mo))
	 , _get_self(
		&wrap_shaped<
			MIRRORED_META_OBJECT(MetaObject),
			meta_meta_object
		>
	)
	{ }

#if RUBBER_POLYMORPHIC_META_OBJECT
	virtual ~meta_object(void)
	{ }
#endif

	inline int uid(void) const
	{
		return _uid;
	}

	friend bool operator == (const meta_object& a, const meta_object& b)
	{
		return a.uid() == b.uid();
	}

	friend bool operator != (const meta_object& a, const meta_object& b)
	{
		return a.uid() != b.uid();
	}

	friend bool operator <= (const meta_object& a, const meta_object& b)
	{
		return a.uid() <= b.uid();
	}

	friend bool operator >= (const meta_object& a, const meta_object& b)
	{
		return a.uid() >= b.uid();
	}

	friend bool operator <  (const meta_object& a, const meta_object& b)
	{
		return a.uid() <  b.uid();
	}

	friend bool operator >  (const meta_object& a, const meta_object& b)
	{
		return a.uid() >  b.uid();
	}

	aux::return_value<meta_meta_object>::type self(void) const
	{
		assert(_get_self != nullptr);
		return _get_self();
	}

	puddle::meta_object_category category(void) const
	{
		return self().category();
	}

#define RUBBER_HELPER_MAKE_MO_IS_X(OBJECT, I, X) \
	inline bool is_## OBJECT(void) const \
	{ \
		return self().is_meta_## OBJECT(); \
	}
MIRROR_FOR_EACH_META_OBJECT(RUBBER_HELPER_MAKE_MO_IS_X, _)
#undef RUBBER_HELPER_MAKE_MO_IS_X

#if RUBBER_POLYMORPHIC_META_OBJECT
	template <typename Interface>
	const Interface& as(void) const
	{
		return dynamic_cast<const Interface&>(*this);
	}
#endif
};

RUBBER_NAMESPACE_END

#endif //include guard

