/**
 *  Copyright 2008-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 */
#ifndef MIRROR_LIBS_EXAMPLES_CLASSES_1011291729_HPP
#define MIRROR_LIBS_EXAMPLES_CLASSES_1011291729_HPP

#include <mirror/meta_class.hpp>
#include <mirror/meta_enum.hpp>

namespace test {

struct A
{
	int a;
};

struct B
{
	bool b;
};

struct C
 : virtual public A
 , virtual public B
{
	char c;
};

struct D
 : virtual public A
 , virtual public B
{
	double d;
};

struct E
 : virtual public C
 , virtual public D
{
	double e;
};

struct F
 : public E
 , virtual public C
 , virtual public D
{
	float f;
};

struct G
{
	short g;
};

struct H
 : public F
 , public G
{
	long h;
};

enum class Ch
{
	null = '\0',
	bell = '\a',
	linefeed = '\n',
	carriage_return = '\r',
	horizontal_tab = '\t',
	formfeed = '\f'
};

class I
{
private:
	static int i;
	MIRROR_FRIENDLY_CLASS(I);
public:
	int get_i(void) const
	{
		return i;
	}

	void set_i(int _i)
	{
		i = _i;
	}
};

int I::i = 0;

struct J : public I
{
	unsigned j;
};

} // namespace test

MIRROR_REG_BEGIN

MIRROR_REG_OBJECT_TAG(hidden)

MIRROR_QREG_GLOBAL_SCOPE_NAMESPACE(test)

MIRROR_REG_CLASS_BEGIN(struct, test, A)
MIRROR_REG_CLASS_END

MIRROR_TAG_TYPE(test::A, (hidden))

MIRROR_REG_CLASS_BEGIN(struct, test, B)
MIRROR_REG_CLASS_END

MIRROR_TAG_TYPE(test::B, (hidden))

MIRROR_REG_CLASS_BEGIN(struct, test, C)
// register base classes
MIRROR_REG_BASE_CLASSES_BEGIN
	MIRROR_REG_BASE_CLASS(virtual, _, test::A),
	MIRROR_REG_BASE_CLASS(virtual, _, test::B)
MIRROR_REG_BASE_CLASSES_END
// register member variables
MIRROR_REG_CLASS_MEM_VARS_BEGIN
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, c)
MIRROR_REG_CLASS_MEM_VARS_END
MIRROR_REG_CLASS_END

MIRROR_REG_CLASS_BEGIN(struct, test, D)
// register base classes
MIRROR_REG_BASE_CLASSES_BEGIN
	MIRROR_REG_BASE_CLASS(virtual, _, test::A),
	MIRROR_REG_BASE_CLASS(virtual, _, test::B)
MIRROR_REG_BASE_CLASSES_END
// register member variables
MIRROR_REG_CLASS_MEM_VARS_BEGIN
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, d)
MIRROR_REG_CLASS_MEM_VARS_END
MIRROR_REG_CLASS_END

MIRROR_REG_CLASS_BEGIN(struct, test, E)
// register base classes
MIRROR_REG_BASE_CLASSES_BEGIN
	MIRROR_REG_BASE_CLASS(virtual, _, test::C),
	MIRROR_REG_BASE_CLASS(virtual, _, test::D)
MIRROR_REG_BASE_CLASSES_END
// register member variables
MIRROR_REG_CLASS_MEM_VARS_BEGIN
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, e)
MIRROR_REG_CLASS_MEM_VARS_END
MIRROR_REG_CLASS_END

MIRROR_REG_CLASS_BEGIN(struct, test, F)
// register base classes
MIRROR_REG_BASE_CLASSES_BEGIN
	MIRROR_REG_BASE_CLASS(_, _, test::E),
	MIRROR_REG_BASE_CLASS(virtual, _, test::C),
	MIRROR_REG_BASE_CLASS(virtual, _, test::D)
MIRROR_REG_BASE_CLASSES_END
// register member variables
MIRROR_REG_CLASS_MEM_VARS_BEGIN
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, f)
MIRROR_REG_CLASS_MEM_VARS_END
MIRROR_REG_CLASS_END

MIRROR_REG_CLASS_BEGIN(struct, test, G)
MIRROR_REG_CLASS_MEM_VARS_BEGIN
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, g)
MIRROR_REG_CLASS_MEM_VARS_END
MIRROR_REG_CLASS_END

MIRROR_REG_CLASS_BEGIN(struct, test, H)
// register base classes
MIRROR_REG_BASE_CLASSES_BEGIN
	MIRROR_REG_BASE_CLASS(_, _, test::F),
	MIRROR_REG_BASE_CLASS(_, _, test::G)
MIRROR_REG_BASE_CLASSES_END
// register member variables
MIRROR_REG_CLASS_MEM_VARS_BEGIN
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, h)
MIRROR_REG_CLASS_MEM_VARS_END
MIRROR_REG_CLASS_END

MIRROR_REG_ENUM_BEGIN(test, Ch)
	MIRROR_REG_ENUM_VALUE(null)
	MIRROR_REG_ENUM_VALUE(bell)
	MIRROR_REG_ENUM_VALUE(linefeed)
	MIRROR_REG_ENUM_VALUE(carriage_return)
	MIRROR_REG_ENUM_VALUE(horizontal_tab)
	MIRROR_REG_ENUM_VALUE(formfeed)
MIRROR_REG_ENUM_END

MIRROR_REG_CLASS_BEGIN(class, test, I)
MIRROR_REG_CLASS_MEM_VARS_BEGIN
	MIRROR_REG_CLASS_MEM_VAR_GET_SET(private, static, int, i, _.get_i(), _.set_i(_i))
MIRROR_REG_CLASS_MEM_VARS_END
MIRROR_REG_CLASS_END

MIRROR_REG_CLASS_BEGIN(struct, test, J)
MIRROR_REG_BASE_CLASSES_BEGIN
	MIRROR_REG_BASE_CLASS(_, _, test::I)
MIRROR_REG_BASE_CLASSES_END
MIRROR_REG_CLASS_MEM_VARS_BEGIN
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, j)
MIRROR_REG_CLASS_MEM_VARS_END
MIRROR_REG_CLASS_END

MIRROR_REG_END

#endif
